@@ -1,12 +1,3 @@
-"""
-Name: Rebound
-Version: 1.1.6
-Author: @shobrook
-Description: Command-line tool that automatically searches Stack Overflow and
-displays results in your terminal when you get a compiler error.
-"""
-
-
 ##########
 ## GLOBALS
 ##########
@@ -24,6 +15,7 @@ from threading import Thread
 import webbrowser
 import time
 from urwid.widget import (BOX, FLOW, FIXED)
+import random
 
 SO_URL = "https://stackoverflow.com"
 
@@ -49,6 +41,40 @@ SCROLL_TO_END = "to end"
 SCROLLBAR_LEFT = "left"
 SCROLLBAR_RIGHT = "right"
 
+USER_AGENTS = [
+    "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; AcooBrowser; .NET CLR 1.1.4322; .NET CLR 2.0.50727)",
+    "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0; en-US)",
+    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Arora/0.3 (Change: 287 c9dfb30)",
+    "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Arora/0.6",
+    "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2pre) Gecko/20070215 K-Ninja/2.1.1",
+    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9) Gecko/20080705 Firefox/3.0 Kapiko/3.0",
+    "Mozilla/5.0 (X11; Linux i686; U;) Gecko/20070322 Firefox/59",
+    "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko Fedora/1.9.0.8-1.fc10 Kazehakase/0.5.6",
+    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11",
+    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.20 (KHTML, like Gecko) Chrome/19.0.1036.7 Safari/535.20",
+    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36',
+    'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36',
+    'Mozilla/5.0 (Windows NT 5.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36',
+    'Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36',
+    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.157 Safari/537.36',
+    'Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36',
+    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36',
+    'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36',
+    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36',
+    'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36',
+    'Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1)',
+    'Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko',
+    'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)',
+    'Mozilla/5.0 (Windows NT 6.1; Trident/7.0; rv:11.0) like Gecko',
+    'Mozilla/5.0 (Windows NT 6.2; WOW64; Trident/7.0; rv:11.0) like Gecko',
+    'Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko',
+    'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0)',
+    'Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko',
+    'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)',
+    'Mozilla/5.0 (Windows NT 6.1; Win64; x64; Trident/7.0; rv:11.0) like Gecko',
+    'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)',
+]
+
 
 ##################
 ## FILE ATTRIBUTES
@@ -81,7 +107,7 @@ def get_error_message(error, language):
         if any(e in error for e in ["KeyboardInterrupt", "SystemExit", "GeneratorExit"]): # Non-compiler errors
             return None
         else:
-            return error.split('\n')[-2][1:]
+            return error.split('\n')[-2].strip()
     elif language == "node":
         return error.split('\n')[4][1:]
     elif language == "go run":
@@ -125,7 +151,7 @@ def read(pipe, funcs):
 def write(get):
     """Pulls output from shared queue and prints to terminal."""
     for line in iter(get, None):
-        sys.stdout.write(line)
+        print(line)
 
 
 ## Main ##
@@ -168,7 +194,7 @@ def execute(command):
     output = ' '.join(output)
     errors = ' '.join(errors)
 
-    if 'java' != command[0] and not os.path.isfile(command[1]): # File doesn't exist, for java, command[1] is a class name instead of a file
+    if "java" != command[0] and not os.path.isfile(command[1]): # File doesn't exist, for java, command[1] is a class name instead of a file
         return (None, None)
     else:
         return (output, errors)
@@ -219,7 +245,7 @@ def get_search_results(soup):
     search_results = []
 
     for result in soup.find_all("div", class_="question-summary search-result"):
-        title_container = result.find_all("div", class_="result-link")[0].find_all("span")[0].find_all("a")[0]
+        title_container = result.find_all("div", class_="result-link")[0].find_all("a")[0]
 
         if result.find_all("div", class_="status answered") != []: # Has answers
             answer_count = int(result.find_all("div", class_="status answered")[0].find_all("strong")[0].text)
@@ -241,7 +267,13 @@ def get_search_results(soup):
 
 def souper(url):
     """Turns a given URL into a BeautifulSoup object."""
-    html = requests.get(url)
+
+    try:
+        html = requests.get(url, headers={"User-Agent": random.choice(USER_AGENTS)})
+    except requests.exceptions.RequestException:
+        sys.stdout.write("\n%s%s%s" % (RED, "Rebound was unable to fetch Stack Overflow results. "
+                                            "Please check that you are connected to the internet.\n", END))
+        sys.exit(1)
 
     if re.search("\.com/nocaptcha", html.url): # URL is a captcha page
         return None
@@ -316,6 +348,7 @@ class Scrollable(urwid.WidgetDecoration):
         self._forward_keypress = None
         self._old_cursor_coords = None
         self._rows_max_cached = 0
+        self._rows_max_displayable = 0
         self.__super.__init__(widget)
 
 
@@ -337,7 +370,7 @@ class Scrollable(urwid.WidgetDecoration):
             fill_height = maxrow - canv_rows
             if fill_height > 0: # Canvas is lower than available vertical space
                 canv.pad_trim_top_bottom(0, fill_height)
-
+        self._rows_max_displayable = maxrow
         if canv_cols <= maxcol and canv_rows <= maxrow: # Canvas is small enough to fit without trimming
             return canv
 
@@ -484,6 +517,9 @@ class Scrollable(urwid.WidgetDecoration):
                 raise RuntimeError("Not a flow/box widget: %r" % self._original_widget)
         return self._rows_max_cached
 
+    @property
+    def scroll_ratio(self):
+        return self._rows_max_cached / self._rows_max_displayable
 
 class ScrollBar(urwid.WidgetDecoration):
     # TODO: Change scrollbar size and color(?)
@@ -505,6 +541,7 @@ class ScrollBar(urwid.WidgetDecoration):
         self.scrollbar_side = side
         self.scrollbar_width = max(1, width)
         self._original_widget_size = (0, 0)
+        self._dragging = False
 
 
     def render(self, size, focus=False):
@@ -596,6 +633,12 @@ class ScrollBar(urwid.WidgetDecoration):
             if is_scrolling_widget(w):
                 return w
 
+    @property
+    def scrollbar_column(self):
+        if self.scrollbar_side == SCROLLBAR_LEFT:
+            return 0
+        if self.scrollbar_side == SCROLLBAR_RIGHT:
+            return self._original_widget_size[0]
 
     def keypress(self, size, key):
         return self._original_widget.keypress(self._original_widget_size, key)
@@ -611,12 +654,25 @@ class ScrollBar(urwid.WidgetDecoration):
         if not handled and hasattr(ow, "set_scrollpos"):
             if button == 4: # Scroll wheel up
                 pos = ow.get_scrollpos(ow_size)
-                ow.set_scrollpos(pos - 1)
-                return True
+                if pos > 0:
+                    ow.set_scrollpos(pos - 1)
+                    return True
             elif button == 5: # Scroll wheel down
                 pos = ow.get_scrollpos(ow_size)
                 ow.set_scrollpos(pos + 1)
                 return True
+            elif col == self.scrollbar_column:
+                ow.set_scrollpos(int(row*ow.scroll_ratio))
+                if event == "mouse press":
+                    self._dragging = True
+                elif event == "mouse release":
+                    self._dragging = False
+            elif self._dragging:
+                ow.set_scrollpos(int(row*ow.scroll_ratio))
+                if event == "mouse release":
+                    self._dragging = False
+
+
 
         return False
 
@@ -687,7 +743,7 @@ class App(object):
 
                 pile = urwid.Pile(self._stylize_question(question_title, question_desc, question_stats) + [urwid.Divider('*')] +
                 interleave(answers, [urwid.Divider('-')] * (len(answers) - 1)))
-                padding = urwid.Padding(ScrollBar(Scrollable(pile)), left=2, right=2)
+                padding = ScrollBar(Scrollable(urwid.Padding(pile, left=2, right=2)))
                 #filler = urwid.Filler(padding, valign="top")
                 linebox = urwid.LineBox(padding)
 
@@ -698,7 +754,7 @@ class App(object):
                     ("menu", u" Q "), ("light gray", u" Quit"),
                 ])
 
-                self.main_loop.widget = urwid.Frame(body=urwid.Overlay(linebox, self.content_container, "center", 85, "middle", 23), footer=menu)
+                self.main_loop.widget = urwid.Frame(body=urwid.Overlay(linebox, self.content_container, "center", ("relative", 60), "middle", 23), footer=menu)
         elif input in ('b', 'B'): # Open link
             url = self._get_selected_link()
 
@@ -752,35 +808,46 @@ def confirm(question):
     prompt = " [Y/n] "
 
     while True:
-        sys.stdout.write(BOLD + CYAN + question + prompt + END)
+        print(BOLD + CYAN + question + prompt + END)
         choice = input().lower()
         if choice in valid:
             return valid[choice]
 
-        sys.stdout.write("Please respond with 'yes' or 'no' (or 'y' or 'n').\n")
+        print("Please respond with 'yes' or 'no' (or 'y' or 'n').\n")
 
 
 def print_help():
     """Prints usage instructions."""
-    sys.stdout.write("%sRebound, V1.1.5a1 - Made by @shobrook%s\n" % (BOLD, END))
-    sys.stdout.write("Command-line tool that automatically searches Stack Overflow and displays results in your terminal when you get a compiler error.")
-    sys.stdout.write("\n\n%sUsage:%s $ rebound %s[file_name]%s\n" % (UNDERLINE, END, YELLOW, END))
-    sys.stdout.write("\n$ python3 %stest.py%s   =>   $ rebound %stest.py%s" % (YELLOW, END, YELLOW, END))
-    sys.stdout.write("\n$ node %stest.js%s     =>   $ rebound %stest.js%s\n\n" % (YELLOW, END, YELLOW, END))
+    print("%sRebound, V1.1.9a1 - Made by @shobrook%s\n" % (BOLD, END))
+    print("Command-line tool that automatically searches Stack Overflow and displays results in your terminal when you get a compiler error.")
+    print("\n\n%sUsage:%s $ rebound %s[file_name]%s\n" % (UNDERLINE, END, YELLOW, END))
+    print("\n$ python3 %stest.py%s   =>   $ rebound %stest.py%s" % (YELLOW, END, YELLOW, END))
+    print("\n$ node %stest.js%s     =>   $ rebound %stest.js%s\n" % (YELLOW, END, YELLOW, END))
+    print("\nIf you just want to query Stack Overflow, use the -q parameter: $ rebound -q %sWhat is an array comprehension?%s\n\n" % (YELLOW, END))
 
 
 ## Main ##
 
 
 def main():
-    if len(sys.argv) == 1:
-        print_help()
-    elif sys.argv[1].lower() == "-h" or sys.argv[1].lower() == "--help":
+    if len(sys.argv) == 1 or sys.argv[1].lower() == "-h" or sys.argv[1].lower() == "--help":
         print_help()
+    elif sys.argv[1].lower() == "-q" or sys.argv[1].lower() == "--query":
+        query = ' '.join(sys.argv[2:])
+        search_results, captcha = search_stackoverflow(query)
+
+        if search_results != []:
+            if captcha:
+                print("\n%s%s%s" % (RED, "Sorry, Stack Overflow blocked our request. Try again in a minute.\n", END))
+                return
+            else:
+                App(search_results) # Opens interface
+        else:
+            print("\n%s%s%s" % (RED, "No Stack Overflow results found.\n", END))
     else:
         language = get_language(sys.argv[1].lower()) # Gets the language name
         if language == '': # Unknown language
-            sys.stdout.write("\n%s%s%s" % (RED, "Sorry, Rebound doesn't support this file type.\n", END))
+            print("\n%s%s%s" % (RED, "Sorry, Rebound doesn't support this file type.\n", END))
             return
 
         file_path = sys.argv[1:]
@@ -798,17 +865,13 @@ def main():
 
             if search_results != []:
                 if captcha:
-                    sys.stdout.write("\n%s%s%s" % (RED, "Sorry, Stack Overflow blocked our request. Try again in a minute.\n", END))
+                    print("\n%s%s%s" % (RED, "Sorry, Stack Overflow blocked our request. Try again in a minute.\n", END))
                     return
                 elif confirm("\nDisplay Stack Overflow results?"):
                     App(search_results) # Opens interface
             else:
-                sys.stdout.write("\n%s%s%s" % (RED, "No Stack Overflow results found.\n", END))
+                print("\n%s%s%s" % (RED, "No Stack Overflow results found.\n", END))
         else:
-            sys.stdout.write("\n%s%s%s" % (CYAN, "No error detected :)\n", END))
+            print("\n%s%s%s" % (CYAN, "No error detected :)\n", END))
 
     return
-
-
-if __name__ == "__main__":
-    main()
